<?php

namespace Drupal\file_upload_secure_validator\Service;

use Drupal\file\Entity\File;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Logger\LoggerChannelTrait;
use Symfony\Component\HttpFoundation\File\MimeType\FileinfoMimeTypeGuesser as SymfonyFileinfoMimeTypeGuesser;

/**
 * A service class for fileinfo-based validation.
 */
class FileUploadSecureValidator {

  // Copies/provides the t() function.
  use StringTranslationTrait;
  // Copies/provides the getLogger() function.
  use LoggerChannelTrait;

  /**
   * File validation function.
   *
   * @param File $file
   *   The file to be uploaded.
   */
  public function validate(File $file) {
    $errors = [];

    // Get mime type from filename.
    $mime_by_filename = $file->getMimeType();
    // Get mime type from fileinfo.
    $symfony_fileinfo_mime_type_guesser = new SymfonyFileinfoMimeTypeGuesser();
    $mime_by_fileinfo = $symfony_fileinfo_mime_type_guesser->guess($file->getFileUri());

    if ($mime_by_filename !== $mime_by_fileinfo) {
      $errors[] = $this->t("There was a problem with this file's extension.");
      $this->getLogger('file_upload_secure_validator')
        ->error("Error while uploading file: MimeTypeGuesser guessed '%mime_by_fileinfo' and fileinfo '%mime_by_filename'",
          ['%mime_by_fileinfo' => $mime_by_fileinfo, '%mime_by_filename' => $mime_by_filename]
        );
    }
    return $errors;
  }

}
